/*
 * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 1.2 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
 
// mDNS-PlatformEnvironment.h needs to ensure that the necessary mDNS types are defined,
// plus whatever additional types are needed to support a particular platform

// To add support for a new target platform with its own networking APIs and types,
// duplicate the "#elif __SOME_OTHER_OS__" section (including its two-line comment
// at the start) and add support for the new target platform in the new section.

#ifndef __mDNSPlatformEnvironment_h
#define __mDNSPlatformEnvironment_h
#if COMPILER_LIKES_PRAGMA_MARK
#pragma once
#endif

#ifdef	__cplusplus
	extern "C" {
#endif

// ***************************************************************************
// Classic Mac (Open Transport) structures

#if (TARGET_API_MAC_OS8 || __MACOS__)

// Headers needed for code on this platform
#include <OpenTptInternet.h>
#include <OpenTptClient.h>

typedef enum
	{
	mOT_Reset = 0,
	mOT_Start,
	mOT_ReusePort,
	mOT_RcvDestAddr,
	mOT_LLScope,
	mOT_AdminScope,
	mOT_Bind,
	mOT_Ready
	} mOT_State;

typedef struct { TOptionHeader h; mDNSIPAddr multicastGroupAddress; mDNSIPAddr InterfaceAddress; } TIPAddMulticastOption;
typedef struct { TOptionHeader h; UInt32 flag; } TSetBooleanOption;

// TOptionBlock is a union of various types.
// What they all have in common is that they all start with a TOptionHeader.
typedef union  { TOptionHeader h; TIPAddMulticastOption m; TSetBooleanOption b; } TOptionBlock;

struct mDNS_PlatformSupport_struct
	{
	EndpointRef ep;
	UInt32 mOTstate;				// mOT_State enum
	TOptionBlock optBlock;
	TOptMgmt optReq;
	long OTTimerTask;
	UInt32 nesting;

	// Platforms that support multi-homing will want a list of HostRecordSets instead of just one
	HostRecordSet hostset;
	};

// ***************************************************************************
// Mac OS X structures

#elif (TARGET_API_MAC_OSX || __MACOSX__)

// Headers needed for code on this platform
#include <SystemConfiguration/SystemConfiguration.h>
#include <IOKit/pwr_mgt/IOPMLib.h>
#include <sys/socket.h>
#include <netinet/in.h>

struct mDNS_PlatformSupport_struct
	{
	CFRunLoopTimerRef  CFTimer;
	SCDynamicStoreRef  Store;
	CFRunLoopSourceRef StoreRLS;
	io_connect_t       PowerConnection;
	io_object_t        PowerNotifier;
	CFRunLoopSourceRef PowerRLS;
	};

// Set this symbol to 1 to do extra debug checks on malloc() and free()
#define MACOSX_MDNS_MALLOC_DEBUGGING 0

#if MACOSX_MDNS_MALLOC_DEBUGGING
extern void *mallocL(char *msg, unsigned int size);
extern void freeL(char *msg, void *x);
#else
#define mallocL(X,Y) malloc(Y)
#define freeL(X,Y) free(Y)
#endif

// ***************************************************************************
// Posix platform structures

#elif defined(MDNS_POSIX)

#include "mDNSPlatformPosix.h"

// ***************************************************************************
// Placeholder for future platforms

#elif __SOME_OTHER_OS__

// ***************************************************************************
// Generic code for Unix-style platforms

#else

#error Other platforms need to make sure that types like UInt16 are defined

#endif
// Emulate Mac OS debugging macros for non-Mac platforms.

#if( !TARGET_OS_MAC )
	#define check(assertion)
	#define check_string( assertion, cstring )
	#define check_noerr(err)
	#define check_noerr_string( error, cstring )
	#define debug_string( cstring )
	#define require( assertion, label )                             	do { if( !(assertion) ) goto label; } while(0)
	#define require_string( assertion, label, string )					require(assertion, label)
	#define require_quiet( assertion, label )							require( assertion, label )
	#define require_noerr( error, label )								do { if( (error) != 0 ) goto label; } while(0)
	#define require_noerr_quiet( assertion, label )						require_noerr( assertion, label )
	#define require_noerr_action( error, label, action )				do { if( (error) != 0 ) { {action;}; goto label; } } while(0)
	#define require_noerr_action_quiet( assertion, label, action )		require_noerr_action( assertion, label, action )
	#define require_action( assertion, label, action )					do { if( !(assertion) ) { {action;}; goto label; } } while(0)
	#define require_action_quiet( assertion, label, action )			require_action( assertion, label, action )
	#define require_action_string( assertion, label, action, cstring )	do { if( !(assertion) ) { {action;}; goto label; } } while(0)
#define	check_errno( ERR, ERRNO )		     \
	do {					     \
		int		localErr;	     \
                                                     \
		localErr = ( int )( ERR );	     \
		if( localErr < 0 ) {		     \
			int		localErrno;  \
						     \
			localErrno = ( ERRNO );	     \
			localErr = ( localErrno != 0 ) ? localErrno : localErr;		\
			lprintf("check_errno: error:%d %s %d\n",localErr, __FILE__, __LINE__);	\
		}		\
	} while( 0 )

#define	require_errno_action( ERR, ERRNO, LABEL, ACTION )	\
do {								\
		int		localErr;			\
								\
		localErr = ( int )( ERR );			\
		if( localErr < 0 ) {				\
			int		localErrno;		\
								\
			localErrno = ( ERRNO );			\
			localErr = ( localErrno != 0 ) ? localErrno : localErr;	\
			lprintf("require_errno_action: error: %d %s %d\n", localErr,  __FILE__, __LINE__);\
			{ ACTION; }				\
			goto LABEL;				\
		}						\
	} while( 0 )

#endif

#ifdef	__cplusplus
	}
#endif

#endif /* __mDNSPlatformEnvironment_h */
